<?php 
class WCCM_GuestToRegistered
{
	var $conversion_result;
	public function __construct()
	{
		add_action( 'add_meta_boxes', array( &$this, 'woocommerce_metaboxes' ) );
		if(is_admin())
			add_action( 'wp_ajax_wccm_convert_guest_to_registered', array( &$this, 'wccm_convert_guest_to_registered' ) );
	}
	
	public function wccm_convert_guest_to_registered()
	{
		if(!wp_verify_nonce( wccm_get_value_if_set($_POST, 'security', ""), 'wccm_admin_customer_export_page' ))
			wp_die();
		
		$this->conversion_result =  "";
		$order = wc_get_order($_POST['order-id']);
		if($order===false)
			wp_die();
		
		$send_email = $_POST['notification-email'] == 'yes' ? true : false;
		$this->convert_guest_to_registered_by_order_id($order, $send_email, false );
		if($this->conversion_result != "")
		{
			echo $this->conversion_result;
		}
		wp_die();
	}
	public function convert_guest_to_registered_by_order_id($order, $send_email, $is_checkout = false )
	{
		global $wccm_order_model, $wccm_customer_model, $wccm_configuration_model;
		$merge_if_existing = $is_checkout ? !WCCM_Options::get_option('do_not_convert_if_email_is_already_associated') : !WCCM_Options::get_option('do_not_convert_if_existing_on_manual_conversion');
		
		//registering
		$conversion_result = $wccm_customer_model->register_new_user($order);
		if($conversion_result['user'] == false)
		{
			$this->conversion_result = $conversion_result['result'];
			return;
		}
		$user = $conversion_result['user'];
		$conversion_result['result'] = $user['already_exists'] && !$merge_if_existing ? __('Email is already associated to another user. Conversion was not performed.', 'woocommerce-customers-manager') : $conversion_result['result'];
		$this->conversion_result = $conversion_result['result'];
		
		//change orders users id reference
		$billing_email = WCCM_Order::get_billing_email($order);
		
		if(!$user['already_exists'] || $merge_if_existing )
		{
			$wccm_order_model->update_order_customer_id($user['id'], $billing_email, $is_checkout ? WCCM_Order::get_order_id($order) : null);
			$wccm_order_model->update_order_download_permission(WCCM_Order::get_order_id($order), $user['id'], $billing_email);
			
			//send notification email
			if($send_email && !$user['already_exists'] && isset($billing_email))
			{
				$mail = new WCCM_Email();
				$subject = $wccm_configuration_model->get_options('email_new_user_subject', __('New account', 'woocommerce-customers-manager'));
				$text = sprintf (__('<br/>User: %s<br/>Password: %s<br/>', 'woocommerce-customers-manager'), $user['user'],$user['password']);
				$user_obj = get_user_by( 'ID', $user['id']);
				$mail->trigger($billing_email, $subject, $text, 'guest_to_restered', is_object($user_obj ) ? $user_obj->ID  : null);
			}
		}
	}
	public function woocommerce_metaboxes()
	{
		add_meta_box( 'woocommerce-customers-manager-guest-to-registered', __('Guest to registered customer conversion', 'woocommerce-customers-manager'), array( &$this, 'woocommerce_guest_to_registered_conversion_box' ), 'shop_order', 'side', 'high');
	}
	function woocommerce_guest_to_registered_conversion_box($post) 
	{
		$order = wc_get_order($post->ID);
		$user_id = $order ? $order->get_user_id() : null;
		$options = WCCM_Options::get_option();
		
		
		if(!apply_filters('wccm_guest_to_registered_order_page_meta_box', true, $order))
		{
			return;
		}
			
		
		if(wccm_get_value_if_set($options, array('admin_order_page','meta_box_disabled_order_statuses', $order->get_status()),'no') == 'yes')
		{
			echo "<h3>".__('Feature not available for current order status', 'woocommerce-customers-manager')."</h3>";
			return;
		}
		
		if(!empty($user_id))
		{
			echo "<h3>".__('Customer already registered', 'woocommerce-customers-manager')."</h3>";
			return;
		}
		$js_options = array(
				'security' => wp_create_nonce('wccm_admin_customer_export_page'),
				);
		wp_register_script('guest-to-registered', WCCM_PLUGIN_PATH.'/js/admin-guest-to-registered-customer.js'); 
		wp_localize_script( 'guest-to-registered', 'wccm', $js_options );
		wp_enqueue_script( 'guest-to-registered');
		wp_enqueue_style('guest-to-registered', WCCM_PLUGIN_PATH.'/css/admin-guest-to-registered-customer.css');
		?>
		<div id="wccm-conversion-box">
			<input type="hidden" id="wccm-conversion-order-id" value="<?php echo $post->ID /* $order->billing_email; */ ?>"></input>
			<label for="wccm-send-notification-email"><?php esc_html_e('Send an email with login info to the customer?', 'woocommerce-customers-manager') ?></label>
			<br/>			
			<select type="checkbox" id="wccm-conversion-notification-email">
				<option value="no" <?php selected(wccm_get_value_if_set($options, array('order_details','send_login_email_option_default_value'),'no'), 'no'); ?>><?php esc_html_e('No', 'woocommerce-customers-manager') ?></option>
				<option value="yes" <?php selected(wccm_get_value_if_set($options, array('order_details','send_login_email_option_default_value'),'no'), 'yes'); ?>><?php esc_html_e('Yes', 'woocommerce-customers-manager') ?></option>
			</select>
			<p><?php wccm_html_escape_allowing_special_tags(__('The plugin will convert guest user to registered. <strong>In case the billing email used by the guest is already associated to a register user, the guest orders will be assigned to that registered user according to the merge options that can be setted in the plugin option menu.</strong>', 'woocommerce-customers-manager'));?></p>
			<br/><br/>
			<input type="submit" id="wccm-conversion-submit-button" class="button" value="<?php esc_html_e('Create customer', 'woocommerce-customers-manager') ?>"></input>
		</div>
		<div id="wccm-conversion-loader">
		</div>
		<div id="wccm-conversion-result">
		<h3><?php esc_html_e('Operation performed', 'woocommerce-customers-manager') ?></h3>
		<p id="wccm-conversion-text-result"></p>
		<a class="button" href="<?php get_edit_post_link( $post->ID ); ?>"><?php esc_html_e('Reload order page to see updated data', 'woocommerce-customers-manager') ?></a>
		</div>
		<?php
	}
	private function random_password($length)
	{
		$string = '';
		$characters = 'abcdefghijklmnopqrstuvwxyz0123456789';
		for ($i = 0; $i < $length; $i++) {
			  $string .= $characters[rand(0, strlen($characters) - 1)];
		}
		return $string;
	}
	
}
?>